@@ -1,4 +1,4 @@
-context("factional.operators")
+context("fractional.operators")
 
 test_that("Operator construction for fractional stationary Matern", {
   x <- seq(from = 0, to = 1, length.out = 51)
@@ -177,4 +177,4 @@ test_that("Operator construction for non-stationary Matern", {
   c1 <- as.vector(Sigma.mult(op1, v))
   c2 <- as.vector(Sigma.mult(op2, v))
   expect_equal(c1, c2, tolerance = 1e-10)
-})
+})
\ No newline at end of file
